@@ -47,13 +47,14 @@
 #include <loader.h>
 #include <e820.h>
 #include <tboot.h>
-#include <lcp2.h>
+#include <lcp3.h>
 #include <elf_defns.h>
 #include <linux_defns.h>
 #include <tb_error.h>
 #include <txt/txt.h>
 #include <mle.h>
 #include <txt/acmod.h>
+#include <cmdline.h>
 
 /* copy of kernel/VMM command line so that can append 'tboot=0x1234' */
 static char *new_cmdline = (char *)TBOOT_KERNEL_CMDLINE_ADDR;
@@ -334,6 +335,8 @@ grow_mb2_tag(loader_ctx *lctx, struct mb2_tag *which, uint32_t how_much)
     /* find the end--we will need it */
     end = (struct mb2_tag *)(lctx->addr + 8);
     end = find_mb2_tag_type(end, MB2_TAG_TYPE_END);
+    if ( end == NULL )
+        return false;
 
     /* How much bigger does it need to be? */
     /* NOTE: this breaks the MBI 2 structure for walking
@@ -550,10 +553,13 @@ static bool adjust_kernel_cmdline(loader_ctx *lctx,
                 printk(TBOOT_ERR"adjust_kernel_cmdline() NULL MB2 cmd\n");
                 return NULL;
             }
-            if (false == 
-                grow_mb2_tag(lctx, cur, 
-                             strlen(new_cmdline) - strlen(cmd->string)))
-                return false;
+            uint32_t new_cmdline_tag_size = 2 * sizeof(uint32_t) + strlen(new_cmdline) + 1;
+            if ( new_cmdline_tag_size > cmd->size ){
+                if (false ==
+                    grow_mb2_tag(lctx, cur,
+                                 (new_cmdline_tag_size - cmd->size)))
+                    return false;
+            }
 
             /* now we're all good, except for fixing up cmd */
             {
@@ -566,7 +572,6 @@ static bool adjust_kernel_cmdline(loader_ctx *lctx,
                 *d = *s;
             }
             // strcpy(cmd->string, cmdbuf);
-            cmd->size = 2 * sizeof(uint32_t) + strlen(cmd->string) + 1;
         }
         return true;
     }
@@ -727,6 +732,8 @@ unsigned long get_mbi_mem_end_mb1(const multiboot_info_t *mbi)
         unsigned int i;
         for ( i = 0; i < mbi->mods_count; i++ ) {
             module_t *p = get_module_mb1(mbi, i);
+            if ( p == NULL )
+                break;
             end = max(end, p->string + strlen((char *)p->string) + 1);
         }
     }
@@ -869,13 +876,14 @@ void fixup_loader_ctx(loader_ctx *lctx, size_t offset)
     return;
 }
 
-static uint32_t get_lowest_mod_start(loader_ctx *lctx)
+static uint32_t get_lowest_mod_start_below_tboot(loader_ctx *lctx)
 {
     uint32_t lowest = 0xffffffff;
     unsigned int mod_count = get_module_count(lctx);
     for ( unsigned int i = 0; i < mod_count; i++ ) {
         module_t *m = get_module(lctx, i);
-        if ( m->mod_start < lowest )
+        if ( m->mod_start < lowest &&
+                below_tboot(m->mod_start))
             lowest = m->mod_start;
     }
 
@@ -904,12 +912,12 @@ move_modules(loader_ctx *lctx)
     if (LOADER_CTX_BAD(lctx))
         return;
 
-    unsigned long lowest = get_lowest_mod_start(lctx);
+    unsigned long lowest = get_lowest_mod_start_below_tboot(lctx);
     unsigned long from = 0;
 
     if ( below_tboot(lowest) )
         from = lowest;
-    else 
+    else
         if ( below_tboot((unsigned long)lctx->addr) )
             from = (unsigned long)lctx->addr;
         else
@@ -948,6 +956,39 @@ module_t *get_module(loader_ctx *lctx, unsigned int i)
     }
 }
 
+static const char *get_boot_loader_name(loader_ctx *lctx)
+{
+    if (LOADER_CTX_BAD(lctx))
+        return NULL;
+    if (lctx->type == MB1_ONLY ){
+        if (((multiboot_info_t *)lctx->addr)->flags & MBI_BTLDNAME)
+            return (char *)((multiboot_info_t *)lctx->addr)->boot_loader_name;
+        return NULL;
+    }
+
+    /* currently must be type 2 */
+    struct mb2_tag *start = (struct mb2_tag *)(lctx->addr + 8);
+    start = find_mb2_tag_type(start, MB2_TAG_TYPE_LOADER_NAME);
+    if (start)
+        return &((struct mb2_tag_string *)start)->string[0];
+
+    return NULL;
+}
+
+static void remove_filename_from_modules_cmdline(loader_ctx *lctx)
+{
+    if (LOADER_CTX_BAD(lctx))
+        return;
+
+    for ( unsigned int i = 0; i < get_module_count(lctx); i++ ) {
+        module_t *m = get_module(lctx, i);
+        char *cmdline = get_module_cmd(lctx, m);
+        const char *adjusted_cmdline = skip_filename(cmdline);
+        if ( adjusted_cmdline != NULL && cmdline != adjusted_cmdline )
+            strncpy(cmdline, adjusted_cmdline, strlen(cmdline));
+    }
+}
+
 static 
 void *remove_first_module(loader_ctx *lctx)
 {
@@ -1164,14 +1205,14 @@ determine_multiboot_type(void *image)
      */
     int result = MB_NONE;
     void *walker;
-    for (walker = image; walker < walker + MULTIBOOT_HEADER_SEARCH_LIMIT;
+    for (walker = image; walker < image + MULTIBOOT_HEADER_SEARCH_LIMIT;
          walker += sizeof(uint32_t)){
         if (*((uint32_t *)walker) == MULTIBOOT_HEADER_MAGIC){
             result += MB1_ONLY;
             break;
         }
     }
-    for (walker = image; walker < walker + MB2_HEADER_SEARCH_LIMIT;
+    for (walker = image; walker < image + MB2_HEADER_SEARCH_LIMIT;
          walker += sizeof(uint64_t)){
         if (*((uint32_t *)walker) == MB2_HEADER_MAGIC){
             result += MB2_ONLY;
@@ -1238,6 +1279,14 @@ bool launch_kernel(bool is_measured_launch)
         
         /* fix for GRUB2, which may load modules into memory before tboot */
         move_modules(g_ldr_ctx);
+
+        /* for GRUB 2, remove the filename in mods' cmdline */
+        const char *loader_name = get_boot_loader_name(g_ldr_ctx);
+        if ( loader_name != NULL &&
+             !strncmp(loader_name, "GNU GRUB", 8) &&
+             strncmp(loader_name, "GNU GRUB 0", 10) )
+            remove_filename_from_modules_cmdline(g_ldr_ctx);
+
     }
     else {
         printk(TBOOT_INFO"assuming kernel is Linux format\n");
