@@ -1,7 +1,7 @@
 /*
  * policy.c: support functions for tboot verification launch
  *
- * Copyright (c) 2006-2010, Intel Corporation
+ * Copyright (c) 2006-2014, Intel Corporation
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,8 +54,8 @@
 #include <integrity.h>
 #include <tpm.h>
 #include <tb_policy.h>
-#include <lcp2.h>
-#include <lcp_hlp.h>
+#include <lcp3.h>
+#include <lcp3_hlp.h>
 #include <cmdline.h>
 #include <txt/config_regs.h>
 #include <txt/mtrrs.h>
@@ -291,7 +291,8 @@ static bool unwrap_lcp_policy(void)
 
             while ( elt ) {
                 /* check element type */
-                if ( elt->type == LCP_POLELT_TYPE_CUSTOM ) {
+                if ( elt->type == LCP_POLELT_TYPE_CUSTOM || 
+                     elt->type == LCP_POLELT_TYPE_CUSTOM2 ) {
                     lcp_custom_element_t *custom =
                         (lcp_custom_element_t *)&elt->data;
 
@@ -310,7 +311,11 @@ static bool unwrap_lcp_policy(void)
 
                 elt = (void *)elt + elt->size;
             }
-            pollist = (void *)pollist + get_policy_list_size(pollist);
+            if ( pollist->version == LCP_TPM12_POLICY_LIST_VERSION )
+                pollist = (void *)pollist + get_tpm12_policy_list_size(pollist);
+            else if ( pollist->version == LCP_TPM20_POLICY_LIST_VERSION )
+                pollist = (void *)pollist + get_tpm20_policy_list_size(
+                        (lcp_policy_list_t2 *)pollist);
         }
     }
 
@@ -349,7 +354,16 @@ tb_error_t set_policy(void)
         printk(TBOOT_DETA"\t:%lu bytes read\n", policy_index_size);
         /* assume lcp policy has been verified by sinit already */
         lcp_policy_t *pol = (lcp_policy_t *)_policy_index_buf;
-        if ( pol->policy_type == LCP_POLTYPE_LIST && unwrap_lcp_policy() ) {
+        if ( pol->version == LCP_DEFAULT_POLICY_VERSION_2 &&
+             pol->policy_type == LCP_POLTYPE_LIST && unwrap_lcp_policy() ) {
+            if ( verify_policy((tb_policy_t *)_policy_index_buf,
+                     calc_policy_size((tb_policy_t *)_policy_index_buf),
+                     true) )
+                goto policy_found;
+        }
+        lcp_policy_t2 *pol2 = (lcp_policy_t2 *)_policy_index_buf;
+        if ( pol2->version == LCP_DEFAULT_POLICY_VERSION &&
+             pol2->policy_type == LCP_POLTYPE_LIST && unwrap_lcp_policy() ) {
             if ( verify_policy((tb_policy_t *)_policy_index_buf,
                      calc_policy_size((tb_policy_t *)_policy_index_buf),
                      true) )
@@ -440,14 +454,21 @@ static bool hash_module(hash_list_t *hl,
             return true;
         }
 
+        uint8_t buf[128];
+
         if ( !g_tpm->hash(g_tpm, 2, base, size, &img_hl) )
             return false;
         for (unsigned int i=0; i<hl->count; i++) {
             for (unsigned int j=0; j<img_hl.count; j++) {
                 if (hl->entries[i].alg == img_hl.entries[j].alg) {
-                    if ( !extend_hash(&hl->entries[i].hash,
-                            &img_hl.entries[j].hash, hl->entries[i].alg) )
+                    copy_hash((tb_hash_t *)buf, &hl->entries[i].hash,
+                            hl->entries[i].alg);
+                    copy_hash((tb_hash_t *)buf + get_hash_size(hl->entries[i].alg),
+                            &img_hl.entries[j].hash, hl->entries[i].alg);
+                    if ( !g_tpm->hash(g_tpm, 2, buf,
+                            2*get_hash_size(hl->entries[i].alg), hl) )
                         return false;
+                    
                     break;
                 }
             }
@@ -789,8 +810,8 @@ static tb_error_t verify_nvindex(tb_policy_entry_t *pol_entry,
         printk(TBOOT_ERR"\t :reading nv index permission failed\n");
         return TB_ERR_NV_VERIFICATION_FAILED;
     }
-    if ( !(attribute & TPM_NV_PER_OWNERWRITE) ) {
-        printk(TBOOT_ERR"\t :nv index should be OWNERWRITE, bad permission\n");
+    if ( !(attribute & (TPM_NV_PER_OWNERWRITE | TPM_NV_PER_AUTHWRITE)) ) {
+        printk(TBOOT_ERR"\t :nv index should be OWNERWRITE or AUTHWRITE, bad permission!\n");
         return TB_ERR_NV_VERIFICATION_FAILED;
     }
 
