@@ -272,7 +272,7 @@ void print_event_2(void *evt, uint16_t alg)
     next += sizeof(uint32_t);
     printk(TBOOT_DETA"\t\t\t       Digest: ");
     print_hex(NULL, (uint8_t *)next, hash_size);
-    next += hash_size/sizeof(uint32_t);
+    next += hash_size;
     data_size = *(uint32_t *)next;
     printk(TBOOT_DETA"\t\t\t         Data: %u bytes", data_size);
     if ( data_size > 4096 ) {
@@ -317,9 +317,9 @@ static void print_evt_log_ptr_elt_2(const heap_ext_data_element_t *elt)
             return;
 
         void *curr, *next;
-        *((uint64_t *)(&curr)) = log_descr->phys_addr +
+        curr = (void *)(unsigned long)log_descr->phys_addr +
                 log_descr->pcr_events_offset;
-        *((uint64_t *)(&next)) = log_descr->phys_addr +
+        next = (void *)(unsigned long)log_descr->phys_addr +
                 log_descr->next_event_offset;
 
         while ( curr < next ) {
@@ -419,7 +419,7 @@ static bool verify_ext_data_elts(const heap_ext_data_element_t elts[],
 }
 
 
-static void print_bios_data(const bios_data_t *bios_data)
+static void print_bios_data(const bios_data_t *bios_data, uint64_t size)
 {
     printk(TBOOT_DETA"bios_data (@%p, %jx):\n", bios_data,
            *((uint64_t *)bios_data - 1));
@@ -432,7 +432,7 @@ static void print_bios_data(const bios_data_t *bios_data)
     printk(TBOOT_DETA"\t num_logical_procs: %u\n", bios_data->num_logical_procs);
     if ( bios_data->version >= 3 )
         printk(TBOOT_DETA"\t flags: 0x%08jx\n", bios_data->flags);
-    if ( bios_data->version >= 4 )
+    if ( bios_data->version >= 4 && size > sizeof(*bios_data) + sizeof(size) )
         print_ext_data_elts(bios_data->ext_data_elts);
 }
 
@@ -488,7 +488,7 @@ bool verify_bios_data(const txt_heap_t *txt_heap)
             return false;
     }
 
-    print_bios_data(bios_data);
+    print_bios_data(bios_data, size);
 
     return true;
 }
@@ -722,7 +722,6 @@ static bool verify_sinit_mle_data(const txt_heap_t *txt_heap)
     sinit_mle_data = get_sinit_mle_data_start(txt_heap);
 
     /* check version */
-    sinit_mle_data = get_sinit_mle_data_start(txt_heap);
     if ( sinit_mle_data->version < 6 ) {
         printk(TBOOT_ERR"unsupported SINIT to MLE data version (%u)\n",
                sinit_mle_data->version);
